////////////////////////////////////////////////////////////////////////
// PongC07.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "Sprites.h"

////////////////////////////////////////////////////////////////////////
//------------------------ Generic functions -------------------------//
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//GetTime()
////////////////////////////////////////////////////////////////////////
unsigned char char3,char4;

unsigned int GetTime()
{
	unsigned int nTime = 0;

	__asm
		CALL #0xBD0D ;KL TIME PLEASE
		PUSH HL
		POP DE
		LD HL, #_char3
		LD (HL), D
		LD HL, #_char4
		LD (HL), E
	__endasm;

	nTime = (char3 << 8) + char4;

	return nTime;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetColor
////////////////////////////////////////////////////////////////////////
void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
	__asm
		ld a, 4 (ix)
		ld b, 5 (ix)
		ld c, b
		call #0xBC32 ;SCR SET INK
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetMode
////////////////////////////////////////////////////////////////////////
void SetMode(unsigned char nMode)
{
	__asm
		ld a, 4 (ix)
		call #0xBC0E ;SCR_SET_MODE
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetBorder
////////////////////////////////////////////////////////////////////////
void SetBorder(unsigned char nColor)
{
	__asm
		ld b, 4 (ix)
		ld c, b
		call #0xBC38 ;SCR_SET_BORDER
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetCursor
////////////////////////////////////////////////////////////////////////
void SetCursor(unsigned char nColum, unsigned char nLine)
{
	__asm
		ld h, 4 (ix)
		ld l, 5 (ix)
		call #0xBB75 ;TXT SET CURSOR
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//PutSprite()
////////////////////////////////////////////////////////////////////////
void PutSprite(unsigned char *pAddress, unsigned char nWidth, unsigned char nHeight, unsigned char *pSprite)
{
	__asm
		LD L, 4(IX) 
		LD H, 5(IX) 
		LD C, 6(IX) 
		LD B, 7(IX)            
		LD E, 8(IX) 
		LD D, 9(IX) 

		_loop_alto:
			PUSH BC
			LD B,C
			PUSH HL
		_loop_ancho:
			LD A,(DE)
			LD (HL),A
			INC DE
			INC HL
			DJNZ _loop_ancho
			POP HL
			LD A,H
			ADD #0x08
			LD H,A
			SUB #0xC0
			JP NC, _sig_linea
			LD BC, #0xC050
			ADD HL,BC
		_sig_linea:
			POP BC
			DJNZ _loop_alto
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//WaitVsync
////////////////////////////////////////////////////////////////////////
void WaitVsync()
{
	__asm
		ld b,#0xf5          ;; PPI port B input
		_wait_vsync:
		in a,(c)            ;; [4] read PPI port B input
    		                ;; (bit 0 = "1" if vsync is active,
    		                ;;  or bit 0 = "0" if vsync is in-active)
		rra                 ;; [1] put bit 0 into carry flag
		jp nc,_wait_vsync   ;; [3] if carry not set, loop, otherwise continue
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//LineHMode2Byte
////////////////////////////////////////////////////////////////////////
void LineHMode2Byte(unsigned int nX, unsigned int nY, unsigned char nColor, unsigned char nBytes)
{
	unsigned char *pAddress = (unsigned char *)0xC000 + ((nY / 8) * 80) + ((nY % 8) * 2048) + (nX / 8);
	
	if(nX % 8 == 0)
	{
		memset(pAddress, nColor ? 0xFF : 0x00, nBytes);
	}
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//---------------- Specific variables and functions-------------------//
////////////////////////////////////////////////////////////////////////

#define NUM_PLAYERS 2
#define SCREEN_WIDTH 640
#define SCREEN_HEIGHT 200

#define SCORE_Y 20
#define SCORE_X_1 ((SCREEN_WIDTH / 2) - 50 - NUMBER_WIDTH / 2)
#define SCORE_X_2 ((SCREEN_WIDTH / 2) + 50 - NUMBER_WIDTH / 2)

#define PLAYER_WIDTH 16
#define PLAYER_HEIGHT 30
#define PLAYER_WIDTH_BYTES 2

struct _tPlayer
{
	unsigned char nY;
	unsigned int nX;
	char nYDir;
}_tPlayer;

struct _tPlayer aPlayer[NUM_PLAYERS];

////////////////////////////////////////////////////////////////////////
//DrawPlayer
////////////////////////////////////////////////////////////////////////
void DrawPlayer(unsigned char nPlayer)
{
	unsigned char nY = 0;
	for(nY = 0; nY < PLAYER_HEIGHT; nY++)
		LineHMode2Byte(aPlayer[nPlayer].nX, aPlayer[nPlayer].nY+ nY, 1, PLAYER_WIDTH_BYTES);
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//MovePlayer
////////////////////////////////////////////////////////////////////////
void MovePlayer(unsigned char nPlayer)
{
	unsigned char nY = 0;
	unsigned char nYToDelete = 0;
	unsigned char nYToDraw = 0;

	int nYNew = aPlayer[nPlayer].nY + aPlayer[nPlayer].nYDir;
	
	if(aPlayer[nPlayer].nYDir > 0)
	{
		if(nYNew + PLAYER_HEIGHT >= SCREEN_HEIGHT)
		{
			nYNew = SCREEN_HEIGHT - PLAYER_HEIGHT - 1;
			aPlayer[nPlayer].nYDir = -4;
		}

		aPlayer[nPlayer].nY = nYNew;		
		nYToDelete = aPlayer[nPlayer].nY - 4;
		nYToDraw = aPlayer[nPlayer].nY + PLAYER_HEIGHT - 4;
	}
	else
	{
		if(nYNew <= 0)
		{
			nYNew = 0;
			aPlayer[nPlayer].nYDir = 4;
		}
		
		aPlayer[nPlayer].nY = nYNew;
		nYToDelete = aPlayer[nPlayer].nY + PLAYER_HEIGHT;
		nYToDraw = aPlayer[nPlayer].nY;
	}
	
	for(nY = 0; nY < 4; nY++)
		LineHMode2Byte(aPlayer[nPlayer].nX, nYToDelete + nY, 0, PLAYER_WIDTH_BYTES);
	
	for(nY = 0; nY < 4; nY++)
		LineHMode2Byte(aPlayer[nPlayer].nX, nYToDraw + nY, 1, PLAYER_WIDTH_BYTES);
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//main
////////////////////////////////////////////////////////////////////////
void main()
{
	unsigned int nFPS = 0;
	unsigned int nTimeLast = 0;
	unsigned int nX = 0;
	unsigned int nY = 0;
	unsigned char *pScore[NUM_PLAYERS];
	unsigned int nCounter = 0;

	pScore[0] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + (SCORE_X_1 / 8);
	pScore[1] = (unsigned char *)0xC000 + ((SCORE_Y / 8) * 80) + ((SCORE_Y % 8) * 2048) + (SCORE_X_2 / 8);

	memset(aPlayer, 0, sizeof(aPlayer));
	aPlayer[0].nY =  100 - PLAYER_HEIGHT / 2;
	aPlayer[1].nY =  100 - PLAYER_HEIGHT / 2;
	aPlayer[0].nX = 8;
	aPlayer[1].nX = (SCREEN_WIDTH - 8) - PLAYER_WIDTH;
	aPlayer[0].nYDir = 4;
	aPlayer[1].nYDir = -4;

	SetMode(2);
	SetBorder(26);
	SetColor(0, 0);
	SetColor(1, 26);
  
  DrawPlayer(0);
  DrawPlayer(1);
  
  
	nTimeLast = GetTime();

	while(1)
	{
		if(GetTime() - nTimeLast >= 300)
		{
			SetCursor(5, 1);
			printf("%u  ", nFPS);

			nTimeLast += 300;
			nFPS = 0;
		}

		WaitVsync();
		PutSprite((unsigned char *)0xC000 + ((nY / 8) * 80) + ((nY % 8) * 2048) + (nX / 8), BALL_WIDTH_BYTES, BALL_HEIGHT, aBallSprite[1]);
		nX = (nX + 8) % (640 - BALL_WIDTH);
		nY = (nY + 4) % (200 - BALL_HEIGHT);
		PutSprite((unsigned char *)0xC000 + ((nY / 8) * 80) + ((nY % 8) * 2048) + (nX / 8), BALL_WIDTH_BYTES, BALL_HEIGHT, aBallSprite[0]);

		MovePlayer(0);
		MovePlayer(1);
		
		nCounter++;
		if(nCounter >= 1000)
			nCounter = 0;
		
		PutSprite(pScore[0], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[nCounter / 100]);
		PutSprite(pScore[1], NUMBER_WIDTH_BYTES, NUMBER_HEIGHT, aNumberSprite[nCounter / 100]);

		nFPS++;
	}
}
////////////////////////////////////////////////////////////////////////